/**
 * @file    IAPV4.h
 *
 *  @date		2016-2019
 *  @copyright	Copyright © GAMEVIL COM2US PLATFORM Inc. All Right Reserved.
 *  @author		hife
 *  @since		4.4.0
 *  \brief		\copydoc 	IAPV4
 */



/**
 * @defgroup IAPV4
 * @{
 */


#ifndef __HIVE_IAPV4_H__
#define __HIVE_IAPV4_H__

#include "HIVE_ResultAPI.h"
#include <memory>

NS_HIVE_BEGIN


/**
 * @brief 결제 마켓 형태
 *
 * @ingroup IAPV4
 * @ingroup hive
 */
enum class IAPV4Type {
    NotSelected = 0
    ,AppleAppStore = 1
    ,GooglePlayStore = 2
    ,HIVELebi = 3
    ,OneStore = 4
    ,AmazonAppStore = 5
    ,SamsungGalaxyStore = 6
    ,HuaweiAppGallery = 7
};

class IAPV4Product;
class IAPV4Receipt;
class IAPV4ReceiptApple;
class IAPV4ReceiptGoogle;
class IAPV4ReceiptLebi;
class IAPV4ReceiptOneStore;
class IAPV4ReceiptAmazon;
class IAPV4ReceiptSamsung;
class IAPV4ReceiptHuawei;


/**
 * \~korean
 * HIVE IAP 는 마켓별 구매 프로세스의 개발 비용을 최소화 할 수 있고 구매 로그를 일관성 있게 수집하여 사업, 개발, 관리 부서에 필요한 정보를 제공할 수 있다.<br/>
 * 현재 HIVE IAP 가 제공하는 결제 마켓은 애플 앱스토어, 구글 플레이 스토어, 원스토어 (https://www.tstore.co.kr), HIVE 러비(Lebi) 을 제공한다.<br/>
 * HIVE 러비(Lebi) 상점은 중국 시장에서 상점을 구성하고 상품을 구매 할 수 있는 시스템으로서 HIVE 에서 자체적으로 운영하고 있다.<br/><br/>
 * \~english
 * HIVE IAP can minimize the development costs of the market-specific purchase process and collect purchase logs consistently to provide information to business, development, and management departments. <br/> * Currently, HIVE IAP supports Apple App Store, Google Play Store, One Store (https://www.tstore.co.kr) and HIVE Lobby (Lebi).<br/>
 * HIVE LIVE (Lebi) store is a system that can organize stores and purchase goods in Chinese market. It is operated by HIVE itself.<br/><br/>
 * \~
 * @since		4.4.0
 * @ingroup IAPV4
 * @author imsunghoon
 */

class HIVESDK_DLLEXPORT IAPV4 {
public:
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 에서 마켓의 구성을 위한 정보 조회 결과 통지
     *
     * @param result		API 호출 결과
     * @param marketIDs	사용 가능한 마켓 정보의 목록<br/>이 마켓 정보의 목록은 게임 클라이언트에서 사용 가능한 마켓을 의미한다.<br/>마켓 목록이 2개 이상이라면 유저가 상점을 선택할 수 있도록 게임에서 유저에게 노출할 수 있는 화면을 제공해야 한다
     *
     *  \~english
     * @brief HIVE IAPV4 Market information request result callback
     *
     * @param result		Result of API call
     * @param marketIDs	사용 가능한 마켓 정보의 목록<br/>
     * This list of market information means the market available on the game client.<br/>
     * If there is more than one market list, you must provide a screen that display the available market list to the user so that the user can select a store. <br/>
     *
     *  \~
     * @see #marketConnect(IAPV4MarketInfoListener)
     *
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result,std::vector<IAPV4Type> const & marketIDs)> onMarketConnect;

    /**
     *  \~korean
     * HIVE IAPV4 상품 정보 조회 결과 통지
     *
     * @param result		    API 호출 결과
     * @param iapV4ProductList  상품 정보
     * @param balance		    HIVE Lebi 마켓의 경우 잔액 정보
     *
     *  \~english
     * HIVE IAPV4 Product Information request callback
     *
     * @param result		    Result of API call
     * @param iapV4ProductList  Product Information
     * @param balance		    Balance information for HIVE Lebi Market If Lebi Market enabled.
     *
     *  \~
     * @see #getProductInfo(IAPV4ProductInfoListener)
     *
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result,std::vector<IAPV4Product> const & iapV4ProductList,unsigned int balance)> onGetProductInfo;


    /**
     *  \~korean
     * @brief 상품 결제 시도 결과 통지<br/>
     * HIVE IAPV4 의 상품 구매는 App Store, Google Store 에 등록된 마켓 상품 (marketPid) 을 요청하게 된다.<br/>
     * additionalInfo 의 경우 구매 요청에 대한 보충 정보로 구매 완료 콜백에 다시 포함되어 전송되지만<br/>
     * 프로모 코드 구매 처럼 앱 외부에서 구매 등의 경우에는 포함될 수 없게 된다.
     *
     * @param result			API 호출 결과
     * @param iapV4Receipt  	구매 성공한 상품 정보 와 additionalInfo 를 포함한 정보
     *
     *  \~english
     * @brief Purchase request result callback<br/>
     * The purchase of HIVE IAPV4 will require a marketPid registered in the App Store and Google Store.<br/>
     * additionalInfo is supplementary information for the purchase request, and is included in the purchase completion callback, <br/>
     * but, in case of purchasing outside of the app, such as a promo code purchase etc., it can not be included in iapV4Receipt.
     *
     * @param result			Result of API call
     * @param iapV4Receipt  	It includes the purchased product information and additionalInfo.
     *
     *  \~
     * @see #purchase(String, String, IAPV4PurchaseListener)
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result,IAPV4Receipt const & iapV4Receipt)> onPurchase;

    /**
     *  \~korean
     * @brief HIVE IAPV4 상품에 대한 미지급된 아이템 지급을 요청 결과 통지<br/>
     * 상품 구매 과정에서 여러가지 상황으로 인하여 아이템 지급을 실패하는 경우가 발생할 수 있다.<br/>
     * 이런 상황을 대비하기 위해서 미 지급된 아이템 지급을 요청 해야 한다.
     *
     * @param result			API 호출 결과
     * @param iapv4ReceiptList  		구매 성공한 상품 정보 와 additionalInfo 를 포함한 정보
     *
     *  \~english
     * @brief HIVE IAPV4 Restore request result callback<br/>
     * It may happen that the item delivery fails due to various situations in the product purchase process.<br/>
     * In order to prepare for this situation, you should request to deliver the item that has not been delivered.
     *
     * @param result			Result of API call
     * @param iapv4ReceiptList  		It includes the purchased product information and additionalInfo
     *
     *  \~
     * @see #restore(IAPV4RestoreListener)
     *
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result,std::vector<std::reference_wrapper<IAPV4Receipt>> const & iapv4ReceiptList)> onRestore;

    /**
     *  \~korean
     * @brief HIVE IAP v4에서 상품에 대한 지급 완료 후 결제 트랜잭션을 종료하기 위한 요청 결과 통지<br/>
     *
     * @param result		API 호출 결과
     * @param marketPid	트랜잭션이 종료된 상품의 marketPid
     *
     *  \~english
     * @brief HIVE IAP v4 Transaction Finish request result callback<br/>
     *
     * @param result		Result of API call
     * @param marketPid	 marketPid of the product whose transaction ended
     *
     *  \~
     * @see #transactionFinish(String, IAPV4TransactionFinishListener)
     *
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result,std::string marketPid)> onTransactionFinish;

    /**
     * @brief
     *
     * @param result		API 호출 결과
     * @param marketPid		Market PID
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result,std::string marketPid)> onCheckPromotePurchase;


    /**
     *  \~korean
     * @brief HIVE IAP v4에서 상품에 대한 지급 완료 후 복수 개의 결제 트랜잭션을 종료하기 위한 요청 결과 통지<br/>
     *
     * @param resultList		트랜잭션의 종료 결과 목록
     * @param marketPidList		트랜잭션이 종료된 상품의 marketPid 목록
     *
     *  \~english
     * @brief HIVE IAP v4 Multiple Transaction Finish request result callback<br/>
     *
     * @param resultList		Result list of the transaction end
     * @param marketPidList		marketPid list of the transaction end
     *
     *  \~
     * @see #transactionMultiFinish(List, IAPV4TransactionMultiFinishListener)
     */
    typedef std::function<void(std::vector<ResultAPI> const & resultList,std::vector<std::string> const & marketPidList)> onTransactionMultiFinish;


    /**
     *  \~korean
     * @brief HIVE 러비 상점일 경우 잔액 확인 결과 통지
     *
     * @param result		API 호출 결과
     * @param balance		HIVE 러비 상점일 경우 잔액
     *
     *  \~english
     * @brief HIVE Lebi Balance request result callback, if it is a Rebi store.
     *
     * @param result		Result of API call
     * @param balance		HIVE Lebi balance
     *
     *  \~
     * @ingroup IAPV4
     */
    typedef std::function<void(ResultAPI const & result, int balance)> onBalance;

    /**
     *  \~korean
     * @brief IAPV4 결재 API 초기화 요청<br/>
     * HIVE IAPV4 의 초기화를 수행하면 결과 통지시 사용 가능한 마켓 정보의 목록을 전달하게 된다.<br/>
     * 이 마켓 정보의 목록은 게임 클라이언트에서 사용 가능한 마켓을 의미한다.
     *
     * @param listener	API 결과 통지
     *
     *  \~english
     * @brief IAPV4 It initialize IAPV4 Purchase module.<br/>
     * When HIVE IAPV4 is initialized, it sends a list of available market information in the result callback.<br/>
     * This list of market information means the market available on the game client.
     *
     * @param listener	Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void marketConnect(onMarketConnect listener);

    /**
     *  \~korean
     * @brief Apple Appstore 에서 상품의 구성을 위한 정보를 조회할 수 있다. <br/>
     *
     * @param listener            API 결과 통지
     *
     *  \~english
     * @brief Apple Appstore It returns information for composition of the product.<br/>
     *
     * @param listener            Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void getMarketProductInfo(std::vector<std::string> const & marketPidList, onGetProductInfo listener);
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 에서 상품의 구성을 위한 정보를 조회할 수 있다. <br/>
     *
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief HIVE IAPV4 It returns information for composition of the product.<br/>
     *
     * @param listener			Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void getProductInfo(onGetProductInfo listener);
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 에서 구독형 상품의 구성을 위한 정보를 조회할 수 있다. <br/>
     *
     * @param handler            API 결과 통지
     *
     *  \~english
     * @brief HIVE IAPV4 It returns information for composition of the product.<br/>
     *
     * @param handler            Result of API call
     *
     *  \~
     * @ingroup IAPV4
     * @since 4.11.6
     */
    static void getSubscriptionProductInfo(onGetProductInfo listener);
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 의 상품 구매는 App Store, Google Store 에 등록된 마켓 상품 (marketPid) 을 요청하게 된다.<br/>
     * additionalInfo 의 경우 구매 요청에 대한 보충 정보로 구매 완료 콜백에 다시 포함되어 전송되지만<br/>
     * 프로모 코드 구매 처럼 앱 외부에서 구매 등의 경우에는 포함될 수 없게 된다.
     *
     * @param marketPid			App Store, Google Store 에 등록된 하나의 상품을 지칭하는 고유 코드
     * @param additionalInfo	구매 요청에 대한 보충 정보를 포함한 문자열로, 개발자가 지정합니다.
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief HIVE IAPV4 The purchase will require a marketPid from the App Store and Google Store.<br/>
     * additionalInfo is supplementary information for the purchase request, and is included in the purchase completion callback, <br/>
     * but, in case of purchasing outside of the app, such as a promo code purchase etc., it can not be included in iapV4Receipt.
     *
     * @param marketPid			Unique code registered on the App Store or Google Store.
     * @param additionalInfo	 A string containing supplementary information about the purchase request, specified by the developer.
     * @param listener			Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void purchase(std::string marketPid,std::string additionalInfo,onPurchase listener);
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 의 상품 구매는 App Store, Google Store 에 등록된 구독 상품 (marketPid) 을 요청하게 된다.<br/>
     * additionalInfo 의 경우 구매 요청에 대한 보충 정보로 구매 완료 콜백에 다시 포함되어 전송되지만<br/>
     * 프로모 코드 구매 처럼 앱 외부에서 구매 등의 경우에는 포함될 수 없게 된다.
     * Android타겟의 경우 구독형 상품을 업그레이드 하거나 다운그레이드를 수행하는 경우 기존에 구독중인 상품의 고유코드 oldMarketPid를 필수로 넣어야한다.
     *
     * @param marketPid            App Store, Google Store 에 등록된 하나의 상품을 지칭하는 새로운 구독 상품의 고유 코드
     * @param oldMarketPid        App Store, Google Store 에 등록된 하나의 상품을 지칭하는 기존 구독 상품의 고유 코드 (Android Only)
     * @param additionalInfo    구매 요청에 대한 보충 정보를 포함한 문자열로, 개발자가 지정합니다. (Android Only)
     * @param listener            API 결과 통지
     *
     *  \~english
     * @brief HIVE IAPV4 The purchase will require a marketPid from the App Store and Google Store.<br/>
     * additionalInfo is supplementary information for the purchase request, and is included in the purchase completion callback, <br/>
     * but, in case of purchasing outside of the app, such as a promo code purchase etc., it can not be included in iapV4Receipt.
     *
     * @param marketPid            Unique code registered on the App Store or Google Store.
     * @param oldMarketPid            Unique code registered on the App Store or Google Store. (Android Only)
     * @param additionalInfo     A string containing supplementary information about the purchase request, specified by the developer. (Android Only)
     * @param listener            Result of API call
     *
     *  \~
     * @ingroup IAPV4
     * @since 4.11.6
     */
    static void purchaseSubscriptionUpdate(std::string marketPid,std::string oldMarketPid,std::string additionalInfo,onPurchase listener);
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 상품에 대한 미지급된 아이템 지급을 요청한다.<br/>
     * 상품 구매 과정에서 여러가지 상황으로 인하여 아이템 지급을 실패하는 경우가 발생할 수 있다.<br/>
     * 이런 상황을 대비하기 위해서 미 지급된 아이템 지급을 요청 해야 한다.<br/>
     *
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief HIVE IAPV4  It restore the item that has not been delivered.<br/>
     * It may happen that the item delivery fails due to various situations in the product purchase process.<br/>
     * In order to prepare for this situation, you should request to deliver the item that has not been delivered.
     *
     * @param listener			API 결과 통지
     *
     *  \~
     * @ingroup IAPV4
     */
    static void restore(onRestore listener);
    
    /**
     *  \~korean
     * @brief HIVE IAPV4 상품에 대한 미지급된 구독 상품 지급을 요청한다.<br/>
     * 상품 구매 과정에서 여러가지 상황으로 인하여 구독 상태를 갱신하지 못할 수 있다.<br/>
     * 구독 상태를 확인할 수 없는 경우, 구독 상태 복구를 요청 해야 한다.<br/>
     *
     * @param listener            API 결과 통지
     *
     *  \~english
     * @brief HIVE IAPV4  It restore the item that has not been delivered.<br/>
     *
     *
     *
     * @param listener            Result of API call
     *
     *  \~
     * @ingroup IAPV4
     * @since 4.11.6
     */
    static void restoreSubscription(onRestore listener);
    
    /**
     *  \~korean
     * @brief HIVE IAP v4에서 지급이 완료된 상품에 대한 구매 영수증에 대한 트랜잭션 완료 처리를 요청한다.<br/>
     * 구매나 복구 기능 사용 후 아이템 지급이 완료된 시점에 이 API를 호출하여 마켓에 구매 완료 처리를 해야한다.<br/>
     *
     * @warning 만약, 지급 완료 후에도 마켓으로의 구매 완료 처리를 하지 않으면, 미지급된 상품으로 남는다.<br/>
     * 미지급된 상품은 transactionFinish API 호출 전까지 계속 복구가 되므로, 영수증 중복 여부를 확인해야 한다.<br/>
     *
     * @param marketPid	지급이 완료되어 트랜잭션 완료 처리를 위한 상품의 고유 ID
     * @param listener	API 결과 통지
     *
     *  \~english
     * @brief HIVE IAP v4 It requests the transaction completion processing for the purchase receipt of the item that have been delivered.<br/>
     * At the completion of item delivery after purchasing or restoring, this API should be called to complete the purchase on the market.<br/>
     *
     * @warning If you do not notify the completion of the purchase to the market after payment has been made, it will be left as an unpaid item..<br/>
     * Since the unpaid items will continue to be recovered until the transactionFinish API call, you should check for duplicate receipts.<br/>
     *
     * @param marketPid	The unique ID of the item for which the transaction is to be completed.
     * @param listener	Result of API call
     *
     *  \~
     * @see IAPV4TransactionFinishListener#transactionFinish(String, IAPV4TransactionFinishListener)
     * @see IAPV4TransactionMultiFinishListener#transactionMultiFinish(List, IAPV4TransactionMultiFinishListener)
     *
     * @ingroup IAPV4
     */
    static void transactionFinish(std::string marketPid,onTransactionFinish listener);

    /**
     *  \~korean
     * @brief HIVE IAP v4에서 지급이 완료된 상품에 대한 구매 영수증에 대한 복수 개의 트랜잭션 완료 처리를 요청한다.<br/>
     * 구매나 복구 기능 사용 후 아이템 지급이 완료된 시점에 이 API를 호출하여 마켓에 구매 완료 처리를 해야한다.<br/>
     *
     * @warning 만약, 지급 완료 후에도 마켓으로의 구매 완료 처리를 하지 않으면, 미지급된 상품으로 남는다.<br/>
     * 미지급된 상품은 transactionFinish API 호출 전까지 계속 복구가 되므로, 영수증 중복 여부를 확인해야 한다.<br/>
     *
     * @param marketPidList	지급이 완료되어 트랜잭션 완료 처리를 위한 상품의 고유 ID 목록
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief HIVE IAP v4 It requests the multiple transaction completion processing for the purchase receipt of the item that have been delivered.<br/>
     * At the completion of item delivery after purchasing or restoring, this API should be called to complete the purchase on the market.<br/>
     *
     * @warning If you do not notify the completion of the purchase to the market after payment has been made, it will be left as an unpaid item..<br/>
     * Since the unpaid items will continue to be recovered until the transactionFinish API call, you should check for duplicate receipts.<br/>
     *
     * @param marketPidList	The list of unique ID of the item for which the transaction is to be completed.
     * @param listener	Result of API call
     *
     *  \~
     * @see IAPV4TransactionMultiFinishListener#transactionMultiFinish(List, IAPV4TransactionMultiFinishListener)
     * @see IAPV4TransactionFinishListener#transactionFinish(String, IAPV4TransactionFinishListener)
     *
     * @ingroup IAPV4
     */
    static void transactionMultiFinish(std::vector<std::string> const & marketPidList,onTransactionMultiFinish listener);

    /**
     *  \~korean
     * @brief 러비 상점이나 구글 상점을 선택하기 위한 창을 띄운다.<br/>
     * 만약 Google Store 와 HIVE 러비(Lebi) 상점을 동시에 사용 가능한 상황이라면 유저가 상점 선택을 할 수 있도록 상점 선택 화면을 노출해야 한다
     *
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief It shows a UI to select a Lebi store or a Google store..<br/>
     * If the Google Store and the HIVE Lebi store are available at the same time, you should expose the store selection screen to allow the user to select a store.
     *
     * @param listener			Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void showMarketSelection(onMarketConnect listener);

    /**
     *  \~korean
     * @brief 러비 상점일 경우 잔액 정보 조회.
     *
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief It request Lebi balance information.
     *
     * @param listener			Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void getBalanceInfo(onBalance listener);

    /**
     *  \~korean
     * @brief 러비 충전 페이지 노출
     *
     * @param listener			API 결과 통지
     *
     *  \~english
     * @brief It shows Lebi refill page.
     *
     * @param listener			Result of API call
     *
     *  \~
     * @ingroup IAPV4
     */
    static void showCharge(onBalance listener);
    
    /**
     @brief Apple AppStore Promote InApp Purchase 동작으로 얻어온 MarketPID를 요청한다.<br/>
     앱의 시작, 앱의 Background -> Foregroud시에 이 API를 호출하여 MarketPID를 얻어오도록 한다.<br/>
     
     얻어온 MarketPID로 구매를 진행하도록 한다.<br/>
     
     @warning iOS 11 이상에서만 지원한다.<br/>
     
     @see #purchase:additionalInfo:handler:
     @see HIVEIAPV4CheckPromotePurchaseHandler
     
     @param listener API 호출 결과 통지
     */
    static void checkPromotePurchase(onCheckPromotePurchase listener);

    /**
     * @brief 현재 선택된 마켓 타입을 반환한다. <br/>
     *
     * 얻어온 마켓 타입으로 구매를 진행하도록 한다.<br/>
     *
     * @see #marketConnect(IAPV4MarketInfoListener)
     *
     * @ingroup IAPV4
     */
    static IAPV4Type getSelectedMarket();

    static void executeEngine(picojson::value jsonParam);
};

/**
 * @brief HIVE IAP 백오피스에서 생성한 상품 정보<br/>
 * 마켓에 등록된 상품(Market Pid) 의 정보를 포함한다.
 *
 * @ingroup IAPV4
 */
class HIVESDK_DLLEXPORT IAPV4Product{
public:
    IAPV4Product();
    IAPV4Product(picojson::value const& jsonParam);

    std::string productType;            ///< \~korean consumable, subscription	 \~english 	consumable, subscription
    std::string marketPid;              ///< \~korean 마켓에 등록된 고유 상품 ID	 \~english 	Unique product ID registered in Market
    std::string currency;				///< \~korean 통화 코드 (ex, KRW / USD)	 \~english 	Currency code (ex, KRW / USD)
    double price;					    ///< \~korean 상품 가격		 \~english 	Product price
    std::string displayPrice;			///< \~korean 상품 가격 문자열 (ex. $100.00)		 \~english 	Product price string (ex. $100.00)
    std::string title;					///< \~korean 상품 타이틀		 \~english 	Item Title
    std::string productDescription;		///< \~korean 상품 설명		 \~english 	Product Description

    //Price before Promotion Sale
    std::string displayOriginalPrice;   /// 세일전 가격 문자열 Google Only
    double originalPrice;               /// 세일전 가격 Google Only
    /**
     * 32-bit PNG (with alpha)
     *
     * Dimensions: 512px by 512px
     *
     * Maximum file size: 1024KB
     */
    std::string iconURL;                /// 상품 아이콘 (512x512) Google Only
    int coinsReward;                    /// 구매시 추가 지급 코인량
};

/**
 * {@link #purchase(String, String, IAPV4PurchaseListener)} 나 {@link #restore(IAPV4RestoreListener)} API 에서 받을 수 있는 결과 정보.<br/>
 * {@link IAPV4Type} 에 따라 {@link IAPV4ReceiptGoogle} 등으로 형변환하여 사용할 수 있다.<br/>
 *
 * @see IAPV4ReceiptGoogle
 * @see IAPV4ReceiptLebi
 * @see IAPV4ReceiptOneStore
 *
 * @ingroup IAPV4
 */
class HIVESDK_DLLEXPORT IAPV4Receipt{
public:
    static std::shared_ptr<IAPV4Receipt> create(picojson::value const& jsonParam);
    IAPV4Receipt();
    virtual ~IAPV4Receipt();
    

    IAPV4Type type;                 ///< \~korean 영수증 마켓 정보	\~english  Receipt Market Information
    IAPV4Product product;           ///< \~korean 구매 성공한 상품 정보	\~english  Purchased product information.
    std::string additionalInfo;     ///< \~korean 구매 요청시 추가로 입력받은 additionalInfo	\~english  AdditionalInfo entered in purchase request
    std::string hiveiapReceipt;     ///< \~korean bypassInfo가 IAP서버 영수증 검증 요청시 보낼 데이터    \~english  Receipt Data to send when requesting validation to IAP server
    std::string bypassInfo;
    
    static IAPV4Type getIAPV4TypeFromJson(picojson::value const& jsonParam);
    
protected:
    IAPV4Receipt(picojson::value const& jsonParam);
};

class HIVESDK_DLLEXPORT IAPV4ReceiptApple : public IAPV4Receipt {
public:
    IAPV4ReceiptApple();
    IAPV4ReceiptApple(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptApple();
    
    IAPV4ReceiptApple& get() const;
};

class HIVESDK_DLLEXPORT IAPV4ReceiptGoogle : public IAPV4Receipt {
    
public:
    IAPV4ReceiptGoogle();
    IAPV4ReceiptGoogle(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptGoogle();
    
    std::string purchaseData;
    std::string signature;
    
    std::string itemType;
    std::string orderId;
    std::string packageName;
    std::string sku;
    std::string developerPayload;
    std::string token;
    std::string vid;
    std::string gameCurrency;
    double gamePrice;
    std::string serverId;
    
    IAPV4ReceiptGoogle& get() const;
};

class HIVESDK_DLLEXPORT IAPV4ReceiptLebi : public IAPV4Receipt {
public:
    IAPV4ReceiptLebi();
    IAPV4ReceiptLebi(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptLebi();
    
    long long tradeno;
    
    long long uid;
    long long vid;
    std::string billItemId;
    
    std::string gameName;
    std::string appId;
    int tradeMoney;
    std::string tradeDate;
    std::string itemName;
    std::string originalJson;
};

class HIVESDK_DLLEXPORT IAPV4ReceiptOneStore : public IAPV4Receipt {
public:
    IAPV4ReceiptOneStore();
    IAPV4ReceiptOneStore(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptOneStore();

    std::string purchaseData;
    std::string signature;

    std::string orderId;
    std::string packageName;
    std::string productId;
    long purchaseTime;
    int acknowledgeState;
    std::string purchaseToken;
    std::string billingKey;
    int purchaseState;
    int recurringState;
    std::string storeCode;
};

class HIVESDK_DLLEXPORT IAPV4ReceiptAmazon : public IAPV4Receipt {
public:
    IAPV4ReceiptAmazon();
    IAPV4ReceiptAmazon(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptAmazon();
    
    std::string userId;
    std::string marketPlace;
    std::string receiptId;
    std::string sku;
    std::string productType;
    std::string purchaseDate;
    std::string cancelDate;
};

class HIVESDK_DLLEXPORT IAPV4ReceiptSamsung : public IAPV4Receipt {
public:
    IAPV4ReceiptSamsung();
    IAPV4ReceiptSamsung(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptSamsung();

    std::string purchaseData;
};

class HIVESDK_DLLEXPORT IAPV4ReceiptHuawei : public IAPV4Receipt {
public:
    IAPV4ReceiptHuawei();
    IAPV4ReceiptHuawei(picojson::value const& jsonParam);
    virtual ~IAPV4ReceiptHuawei();
    
    std::string purchaseData;
    std::string signature;

    std::string orderId;
    std::string productId;
    std::string priceType;
};
NS_HIVE_END


#endif /* __HIVE_IAPV4_H__ */
