/**
 * @file    C2SModuleSecureVariable.cpp
 
 * @brief   변수 암호화 모듈의 소스
 
 * @author  Byeong Yeol Jeon
 
 * @date    2014/04/28
 * @ingroup iOS Android Cocos2d-x Protection
 */
#include "C2SModuleSecureVariable.h"
#include "cscrc32.h"
#include <cstdlib>
#include <pthread.h>
#include <time.h>
#include <string.h>
/**
 * @class C2SModuleAutoLock
 
 * @brief constructor에서 락을 걸고, destructor에서 락을 해제하는 클래스
 * @author Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x Protection
 */
class C2SModuleAutoLock
{
public:
    /**
     * @brief constructor. 락을 건다.
     
     * @param[in] arg pthread_mutex_t의 핸들
     
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleAutoLock(pthread_mutex_t& arg) : mutex(arg)
    {
        pthread_mutex_lock(&mutex);
    }
    /**
     * @brief destructor. 락을 해제한다.
     
     * @author Byeong Yeol Jeon
     
     */
    ~C2SModuleAutoLock()
    {
        pthread_mutex_unlock(&mutex);
    }
private:
    /** copy prevent */
    C2SModuleAutoLock(const C2SModuleAutoLock&);
    /** copy prevent */
    C2SModuleAutoLock& operator=(const C2SModuleAutoLock&);
    
    /** constructor에서 입력받은 핸들의 레퍼런스 */
    pthread_mutex_t& mutex;
};
/**
 * @class C2SModuleSecureVariableImpl
 
 * @brief 변수 암호화의 실제 구현부
 
 * @author Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x Protection
 */
class C2SModuleSecureVariableImpl
{
public:
    /**
     * @brief default constructor
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleSecureVariableImpl() : realData(NULL), prev(NULL), dataLength(0), crcKey(0)
    {
        cryptKey[0] = rand()%16;
        cryptKey[1] = rand()%16;
        
        // 락을 건다.
        C2SModuleAutoLock lockObj(List::GetInstance()->mutex);
        
        // 리스트의 맨 앞에 현재 인스턴스를 삽입한다.
        C2SModuleSecureVariableImpl* head = List::GetInstance()->head;
        if(NULL != head)
        {
            next = head->next;
            head->prev = this;
        }
        else
        {
            next = NULL;
        }
        List::GetInstance()->head = this;
    }
    /**
     * @brief default destructor
     
     * @author Byeong Yeol Jeon
     
     */
    ~C2SModuleSecureVariableImpl()
    {
        // 락을 건다.
        C2SModuleAutoLock lockObj(List::GetInstance()->mutex);
        
        // 현재 인스턴스를 리스트에서 제거한다.
        if(NULL != prev)
            prev->next = next;
        
        if(NULL != next)
            next->prev = prev;
        
        delete realData;
    }
    /**
     * @brief 저장된 데이터를 구한다.
     
     * @param[in] data 데이터가 저장될 버퍼
     * @param[in] data 데이터가 저장될 버퍼의 길이
     * @return 실행 중 발생한 에러의 코드
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleErrorCode Get(void* data, uint32_t length)
    {
        C2SModuleAutoLock lockObj(List::GetInstance()->mutex);
        
        return GetInner(data, length);
    }
    /**
     * @brief 데이터를 저장한다.
     
     * @param[in] data 저장할 데이터
     * @param[in] length 저장할 데이터의 길이
     * @return 실행 중 발생한 에러의 코드
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleErrorCode Set(const void* data, uint32_t length)
    {
        if(NULL == data || 0 == length)
        {
            return C2SModuleErrorCode_InvalidArg;
        }
        
        uint8_t* temp  = new uint8_t[length+4];
        
        if(NULL == temp)
        {
            return C2SModuleErrorCode_MemoryExceed;
        }
        
        C2SModuleAutoLock lockObj(List::GetInstance()->mutex);
        
        return SetInner(data, length, temp);
    }
    /**
     * @brief 현재 데이터를 입력 데이터로부터 복사한다.
     
     * @param[in] src 입력 데이터
     
     * @return 실행 중 발생한 에러코드
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleErrorCode Copy(C2SModuleSecureVariableImpl* src)
    {
        if(NULL == src)
        {
            return C2SModuleErrorCode_InvalidArg;
        }
        
        C2SModuleAutoLock lockObj(List::GetInstance()->mutex);
        
        return CopyInner(src);
    }
    
private:
    /**
     * @brief CRC를 계산한다.
     * @return true일 경우 성공. false일 경우 실패
     * @author Byeong Yeol Jeon
     
     */
    bool CalcCRC()
    {
        return List::GetInstance()->crc.update(crcKey, List::GetInstance()->crc_accum, realData, dataLength);
    }
    /**
     * @brief 입력받은 crc값으로 crc를 체크한다.
     * @param[in] crcCheck 체크할 값
     * @return true일 경우 성공. false일 경우 실패
     * @author Byeong Yeol Jeon
     
     */
    bool CalcCRC(uint32_t crcCheck)
    {
        return List::GetInstance()->crc.update(crcCheck, List::GetInstance()->crc_accum, realData, dataLength + 4);
    }
    /**
     * @brief Get의 내부 처리 함수
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleErrorCode GetInner(void* data, uint32_t length)
    {
        if(NULL == data || 0 == length || length != dataLength)
        {
            return C2SModuleErrorCode_InvalidArg;
        }
        
        realData[dataLength  ] = (crcKey & 0xFF000000L) >> 24;
        realData[dataLength+1] = (crcKey & 0x00FF0000L) >> 16;
        realData[dataLength+2] = (crcKey & 0x0000FF00L) >> 8;
        realData[dataLength+3] = (crcKey & 0x000000FFL);
        
        uint32_t crcCheck = 0;
        CalcCRC(crcCheck);
        if(0 != crcCheck)
        {
            return C2SModuleErrorCode_Broken;
        }
        
        uint8_t* dst = reinterpret_cast<uint8_t*>(data) + length - 1;
        const uint8_t* src = realData;
        
        do
        {
            *dst = (((*src&0x0F)^cryptKey[1])<<4)|(((*src&0xF0)>>4)^cryptKey[0]);
            --dst;
            ++src;
        } while(0 != --length);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Set의 내부 처리함수
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleErrorCode SetInner(const void* data, uint32_t length, uint8_t* newData)
    {
        delete realData;
        
        realData = newData;
        dataLength = length;
        
        uint8_t* dst = realData + length - 1;
        const uint8_t* src = reinterpret_cast<const uint8_t*>(data);
        
        do
        {
            *dst = ((((*src)&0x0F)^cryptKey[0])<<4)|((((*src)&0xF0)>>4)^cryptKey[1]);
            --dst;
            ++src;
        } while(0 != --length);
        
        CalcCRC();
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Copy의 내부 처리함수
     
     * @author Byeong Yeol Jeon
     
     */
    C2SModuleErrorCode CopyInner(C2SModuleSecureVariableImpl* src)
    {
        if(NULL == src->realData || 0 == src->dataLength)
            return C2SModuleErrorCode_InvalidArg;
        
        if(NULL == realData || dataLength != src->dataLength)
        {
            delete  realData;
            realData = new uint8_t[src->dataLength+4];
            dataLength = src->dataLength;
        }
        
        memcpy(realData, src->realData, dataLength);
        crcKey = src->crcKey;
        cryptKey[0] = src->cryptKey[0];
        cryptKey[1] = src->cryptKey[1];
        
        return C2SModuleErrorCode_Success;
    }
    
    /** */
    uint8_t* realData;
    /** */
    uint32_t dataLength;
    /** */
    uint32_t crcKey;
    /** */
    C2SModuleSecureVariableImpl* prev;
    /** */
    C2SModuleSecureVariableImpl* next;
    /** */
    uint8_t cryptKey[2];
    /**
     * @class List
     
     * @brief 내부적으로 암호화 변수들의 리스트를 관리하는 클래스
     * @author Byeong Yeol Jeon
     
     */
    class List
    {
    public:
        /**
         * @brief 싱글턴 인스턴스를 구한다.
         * @return List의 싱글턴 인스턴스
         
         * @author Byeong Yeol Jeon
         
         */
        static List* GetInstance()
        {
            static List instance;
            return &instance;
        }
        /**
         * @brief destructor
         * @author Byeong Yeol Jeon
         
         */
        ~List()
        {
            //락을 건다.
        	pthread_mutex_lock(&mutex);
            
            // 모든 변수를 제거한다.
        	while(NULL != head)
        	{
        		C2SModuleSecureVariableImpl* target = head;
        		head = head->next;
        		delete target;
        	}
            
            // 락을 해제하고 제거한다.
        	pthread_mutex_unlock(&mutex);
            pthread_mutex_destroy(&mutex);
        }
        /** 링크드 리스트의 헤드 인스턴스 */
        C2SModuleSecureVariableImpl* head;
        /** crc 테이블 객체 */
        CSCRC32 crc;
        /** 누적된 crc값 */
        uint32_t crc_accum;
        /** 락 오브젝트 */
        pthread_mutex_t mutex;
    private:
        /**
         * @brief constructor
         
         */
        List() : head(NULL)
        {
            pthread_mutex_init(&mutex, NULL);
            srand((uint32_t)time(NULL));
            crc_accum = rand()%100;
        }
    };
    /** copy prevent */
    C2SModuleSecureVariableImpl(const C2SModuleSecureVariableImpl&);
    /** copy prevent */
    C2SModuleSecureVariableImpl& operator=(const C2SModuleSecureVariableImpl&);
};

#if __cplusplus
extern "C" {
#endif
    C2SModuleErrorCode C2SModuleSecureVariableCreate(C2SModuleSecureId* outSecureId)
    {
        if(NULL == outSecureId)
            return C2SModuleErrorCode_InvalidArg;
        
        C2SModuleSecureVariableImpl* instance = new C2SModuleSecureVariableImpl();
        
        *outSecureId = reinterpret_cast<C2SModuleSecureId>(instance);
        
        return C2SModuleErrorCode_Success;
    }
    
    C2SModuleErrorCode C2SModuleSecureVariableGet(C2SModuleSecureId secureId, void* data, uint32_t length)
    {
        C2SModuleSecureVariableImpl* value = reinterpret_cast<C2SModuleSecureVariableImpl*>(secureId);
        
        if(NULL == value)
            return C2SModuleErrorCode_InvalidArg;
        
        return value->Get(data, length);
    }
    
    C2SModuleErrorCode C2SModuleSecureVariableSet(C2SModuleSecureId secureId, const void* data, uint32_t length)
    {
        C2SModuleSecureVariableImpl* value = reinterpret_cast<C2SModuleSecureVariableImpl*>(secureId);
        
        if(NULL == value)
            return C2SModuleErrorCode_InvalidArg;
        
        return value->Set(data, length);
    }
    
    C2SModuleErrorCode C2SModuleSecureVariableCopy(C2SModuleSecureId secureId, C2SModuleSecureId srcId)
    {
        C2SModuleSecureVariableImpl* dstValue = reinterpret_cast<C2SModuleSecureVariableImpl*>(secureId);
        C2SModuleSecureVariableImpl* srcValue = reinterpret_cast<C2SModuleSecureVariableImpl*>(srcId);
        
        if(NULL == dstValue || NULL == srcValue)
            return C2SModuleErrorCode_InvalidArg;
        
        return dstValue->Copy(srcValue);
    }
    
    C2SModuleErrorCode C2SModuleSecureVariableRelease(C2SModuleSecureId* secureId)
    {
        C2SModuleSecureVariableImpl** ppImpl = reinterpret_cast<C2SModuleSecureVariableImpl**>(secureId);
        
        if(NULL == ppImpl || NULL == *ppImpl)
            return C2SModuleErrorCode_InvalidArg;
        
        delete *ppImpl;
        *ppImpl = NULL;
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief 8bit signed를 읽는다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in, out] data 저장될 변수의 주소
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet8(C2SModuleSecureId secureId,
                                                   int8_t* data)
    {
        return C2SModuleSecureVariableGet(secureId, data, sizeof(int8_t));
    }
    /**
     * @brief 16bit signed를 읽는다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in, out] data 저장될 변수의 주소
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet16(C2SModuleSecureId secureId,
                                                    int16_t* data)
    {
        return C2SModuleSecureVariableGet(secureId, data, sizeof(int16_t));
    }
    /**
     * @brief 32bit signed를 읽는다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in, out] data 저장될 변수의 주소
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet32(C2SModuleSecureId secureId,
                                                    int32_t* data)
    {
        return C2SModuleSecureVariableGet(secureId, data, sizeof(int32_t));
    }
    /**
     * @brief 32bit float를 읽는다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in, out] data 저장될 변수의 주소
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet32F(C2SModuleSecureId secureId,
                                                     float* data)
    {
        return C2SModuleSecureVariableGet(secureId, data, sizeof(float));
    }
    /**
     * @brief 64bit signed를 읽는다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in, out] data 저장될 변수의 주소
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet64(C2SModuleSecureId secureId,
                                                    int64_t* data)
    {
        return C2SModuleSecureVariableGet(secureId, data, sizeof(int64_t));
    }
    /**
     * @brief 64bit float를 읽는다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in, out] data 저장될 변수의 주소
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableGet64F(C2SModuleSecureId secureId,
                                                     double* data)
    {
        return C2SModuleSecureVariableGet(secureId, data, sizeof(double));
    }
    /**
     * @brief 8bit signed를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장될 데이터
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet8(C2SModuleSecureId secureId,
                                                   int8_t data)
    {
        return C2SModuleSecureVariableSet(secureId, &data, sizeof(int8_t));
    }
    /**
     * @brief 16bit signed를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장될 데이터
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet16(C2SModuleSecureId secureId,
                                                    int16_t data)
    {
        return C2SModuleSecureVariableSet(secureId, &data, sizeof(int16_t));
    }
    /**
     * @brief 32bit signed를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장될 데이터
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet32(C2SModuleSecureId secureId,
                                                    int32_t data)
    {
        return C2SModuleSecureVariableSet(secureId, &data, sizeof(int32_t));
    }
    /**
     * @brief 32bit float를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장될 데이터
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet32F(C2SModuleSecureId secureId,
                                                     float data)
    {
        return C2SModuleSecureVariableSet(secureId, &data, sizeof(float));
    }
    /**
     * @brief 64bit signed를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장될 데이터
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet64(C2SModuleSecureId secureId,
                                                    int64_t data)
    {
        return C2SModuleSecureVariableSet(secureId, &data, sizeof(int64_t));
    }
    /**
     * @brief 64bit float를 암호화하여 저장한다.
     * @param[in] secureId 암호화 오브젝트의 핸들
     * @param[in] data 저장될 데이터
     * @return 실행시 발생한 에러의 코드
     * @ingroup Protection Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSecureVariableSet64F(C2SModuleSecureId secureId,
                                                     double data)
    {
        return C2SModuleSecureVariableSet(secureId, &data, sizeof(double));
    }
#if __cplusplus
}
#endif
