/**
 * @file    adapt_tuple.h
 
 * @brief   튜플을 메타템플릿 프로그래밍으로 파싱하는 클래스의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/30
 */
#ifndef _ADAPT_TUPLE_H_
#define _ADAPT_TUPLE_H_

#if __cplusplus >= 201103L

#define ADAPT_TUPLE 1

#include <tuple>

#ifndef _NOEXCEPT
#define _NOEXCEPT noexcept(true)
#endif

namespace C2SModule_Cocos2dx {
    /**
     * @class TupleMapIterator
     
     * @brief 튜플을 순회하며 데이터를 맵에 넣는다.
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    template<class MapType, class ArrayType, int32_t index>
    class TupleMapIterator {
    public:
        /**
         * @brief predicator
         
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] arg 튜플
         * @author Byeong Yeol Jeon
         
         */
        template<typename... T>
        static void Pred(MapType& parent, const std::tuple<T...>& arg) _NOEXCEPT
        {
            // 맵에 키 밸류 페어로 데이터가 입력되어야하므로 체크한다.
            typedef std::tuple< T... > arg_type;
            static_assert( 2 <= std::tuple_size< arg_type >::value, "too few arguments" );
            static_assert( 0 == (std::tuple_size< arg_type >::value & 1), "key-value pair mismatch" );
            
            typedef typename std::tuple_element<index-1, arg_type>::type value_type;
            
            // 튜플의 마지막 밸류와 키를 파싱한다.
            TupleMapIterator::PredInner(parent, std::get<index-2>(arg), std::get<index-1>(arg));
            
            // 그다음 튜플 엘레먼트를 탐색
            TupleMapIterator<MapType, ArrayType, index - 2>::Pred(parent, arg);
        }
        
    private:
        /**
         * @brief inner predicator. 맵의 키 위치에 맵을 값으로 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        template<typename... T>
        static void PredInner(MapType& parent, const char key[],
                              MapType& arg) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, arg);
        }
        /**
         * @brief inner predicator. 맵의 키 위치에 배열을 값으로 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        template<typename... T>
        static void PredInner(MapType& parent, const char key[],
                              ArrayType& arg) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, arg);
        }
        /**
         * @brief inner predicator. 맵의 키 위치에 bool값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     bool value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        /**
         * @brief inner predicator. 맵의 키 위치에 8bit signed값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     int8_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        /**
         * @brief inner predicator. 맵의 키 위치에 8bit unsigned값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     uint8_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 16bit signed값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     int16_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 16bit unsigned값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     uint16_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 32bit signed값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     int32_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 32bit unsigned값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     uint32_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 64bit signed값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     int64_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 64bit unsigned값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     uint64_t value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 문자열값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     const char value[]) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 stl string을 문자열로 변환한 값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     const std::string& value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value.c_str());
        }
        
        /**
         * @brief inner predicator. 맵의 키 위치에 raw image값을 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     const C2SModuleImage& value) _NOEXCEPT
        {
            if(NULL == key)
            {
                return;
            }
            
            parent.Set(key, value);
        }
        
        /*
        inline static void PredInner(MapType& parent, const char key[],
                                     void* value) _NOEXCEPT
        {
            if(NULL == key || NULL == value)
            {
                return;
            }
#if __OBJC__
            //[parent setObject:[C2SModuleUserPtr Convert:value] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
#endif // __OBJC__
        }
        */
        
#if __OBJC__
        /**
         * @brief inner predicator. 맵의 키 위치에 objc 인스턴스를 값으로 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     id value) _NOEXCEPT
        {
            if(NULL == key || NULL == value)
            {
                return;
            }
            
            parent.Set(key, value);
        }
#elif __ANDROID__
        /**
         * @brief inner predicator. 맵의 키 위치에 android 인스턴스를 값으로 넣는다.
         * @param[in] parent 데이터를 입력받는 맵
         * @param[in] key 맵에서 값이 입력될 키
         * @param[in] value 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(MapType& parent, const char key[],
                                     jobject value) _NOEXCEPT
        {
            if(NULL == key || NULL == value)
            {
                return;
            }
            
            parent.Set(key, value);
        }
#endif
        /**
         * @brief inner predicator. 지원되지 않는 형식을 스킵한다.
         * @author Byeong Yeol Jeon
         */
        template<typename ... T>
        static void PredInner(T&&...)
        {
        }
    };
    
    /**
     * @brief 튜플을 순회하여 마지막에 도달했을 때의 구현
     
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    template<class MapType, class ArrayType>
    struct TupleMapIterator<MapType, ArrayType, 0> {
    public:
        /**
         * @brief predicator.
         * @author Byeong Yeol Jeon
         */
        template<typename ... T>
        static void Pred(T&&...)
        {
        }
        
    private:
        /**
         * @brief inner predicator.
         * @author Byeong Yeol Jeon
         */
        template<typename ... T>
        static void PredInner(T&&...)
        {
        }
    };
    
    /**
     * @class TupleArrayIterator
     
     * @brief 튜플을 순회하며 데이터를 배열에 추가한다.
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    template<class MapType, class ArrayType, int32_t index>
    class TupleArrayIterator {
    public:
        /**
         * @brief predicator
         
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 튜플
         * @author Byeong Yeol Jeon
         
         */
        template<typename... T>
        static void Pred(ArrayType& parent, const std::tuple<T...>& arg) _NOEXCEPT
        {
            // 튜플의 타입을 별칭으로 정의
            typedef std::tuple< T... > arg_type;
            typedef typename std::tuple_element<index-1, arg_type>::type value_type;
            
            // 마지막 인덱스의 튜플 인자를 inner pred해준다.
            TupleArrayIterator::PredInner(parent, std::get<index-1>(arg));
            
            // 인덱스를 하나 감소시켜 다시 pred를 수행한다.
            TupleArrayIterator<MapType, ArrayType, index - 1>::Pred(parent, arg);
        }
        
    private:
        /**
         * @brief inner predicator. 배열에 맵을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        template<typename... T>
        static void PredInner(ArrayType& parent, MapType& arg) _NOEXCEPT
        {
            parent.Add(arg);
        }
        /**
         * @brief inner predicator. bool 값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, bool value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 8bit signed값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, int8_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 8bit unsigned값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, uint8_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 16bit signed값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, int16_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 8bit unsigned값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, uint16_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 32bit signed값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, int32_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 32bit unsigned값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, uint32_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 64bit signed값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, int64_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. 64bit unsigned값을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, uint64_t value) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. c문자열을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent, const char value[]) _NOEXCEPT
        {
            parent.Add(value);
        }
        /**
         * @brief inner predicator. stl string을 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent,
                                     const std::string& value) _NOEXCEPT
        {
            parent.Add(value.c_str());
        }
        
#if __OBJC__
        /**
         * @brief inner predicator. objc 인스턴스를 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent,
                                     id value) _NOEXCEPT
        {
            parent.Add(value);
        }
#elif __ANDROID__
        /**
         * @brief inner predicator. android 인스턴스를 원소로 추가한다.
         * @param[in] parent 데이터를 입력받는 배열
         * @param[in] arg 입력될 값
         * @author Byeong Yeol Jeon
         */
        inline static void PredInner(ArrayType& parent,
                                     jobject value) _NOEXCEPT
        {
            parent.Add(value);
        }
#endif
        /**
         * @brief inner predicator. 허용하지 않는 타입일 경우 아무것도 처리하지 않는다.
         * @author Byeong Yeol Jeon
         */
        template<typename ... T>
        static void PredInner(T&&...)
        {
        }
    };
    
    /**
     * @brief 튜플을 순회하여 인덱스의 마지막에 도달했을 때의 구현
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    template<class MapType, class ArrayType>
    class TupleArrayIterator<MapType, ArrayType, 0> {
    public:
        /**
         * @brief predicator.
         * @author Byeong Yeol Jeon
         */
        template<typename ... T>
        static void Pred(T&&...)
        {
        }
        
    private:
        /**
         * @brief inner predicator.
         * @author Byeong Yeol Jeon
         */
        template<typename ... T>
        static void PredInner(T&&...)
        {
        }
    };
}
#endif

#endif
